----------------------------------------------------------------------------------
-- MixerWagon AF + PF
-- Specialization for Mixerwagon alternates AF + PF
--
-- @author TwistedGetsu
-- @date 23/01/2019
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
----------------------------------------------------------------------------------

customSpec = {}
customSpec.title = "Custom Spec MixerWagon"
customSpec.author = "TwistedGetsu"
customSpec.modDirectory = g_currentModDirectory

function customSpec.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations)-- Specialization requirement (optional)
		and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);-- Specialization requirement (optional)
end;

function customSpec.registerEventListeners(vehicleType)
	for _, functionName in pairs( { "onPreLoad", "onLoad" } ) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, customSpec)
	end
end
  
function customSpec:onPreLoad(savegame)
	MixerWagon.onLoad = Utils.overwrittenFunction(MixerWagon.onLoad, customSpec.onNilLoad)
end

function customSpec:onNilLoad(savegame)
--nil
end

function customSpec:onLoad(savegame)

	print("customSpec Script Loaded & Working");

    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonBaleTrigger#index", "vehicle.mixerWagon.baleTrigger#node") --FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagon.baleTrigger#index", "vehicle.mixerWagon.baleTrigger#node") --FS19 to FS19

    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonPickupStartSound", "vehicle.turnOnVehicle.sounds.start") --FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonPickupStopSound", "vehicle.turnOnVehicle.sounds.stop") --FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonPickupSound", "vehicle.turnOnVehicle.sounds.work") --FS17 to FS19

    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonRotatingParts.mixerWagonRotatingPart#type", "vehicle.mixerWagon.mixAnimationNodes.animationNode", "mixerWagonMix") --FS17 to FS19
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonRotatingParts.mixerWagonRotatingPart#type", "vehicle.mixerWagon.pickupAnimationNodes.animationNode", "mixerWagonPickup") --FS17 to FS19
        
    XMLUtil.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.mixerWagonRotatingParts.mixerWagonScroller", "vehicle.mixerWagon.pickupAnimationNodes.pickupAnimationNode") --FS17 to FS19
        
    local spec = self.spec_mixerWagon
        
    if self.isClient then
        spec.mixAnimationNodes = g_animationManager:loadAnimations(self.xmlFile, "vehicle.mixerWagon.mixAnimationNodes", self.components, self, self.i3dMappings)
        spec.pickupAnimationNodes = g_animationManager:loadAnimations(self.xmlFile, "vehicle.mixerWagon.pickupAnimationNodes", self.components, self, self.i3dMappings)
    end

    if self.isServer then
        spec.baleTriggerNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mixerWagon.baleTrigger#node"), self.i3dMappings)
        if spec.baleTriggerNode ~= nil then
            addTrigger(spec.baleTriggerNode, "mixerWagonBaleTriggerCallback", self)
        end
    end

    spec.activeTimerMax = 5000
    spec.activeTimer = 0

    spec.fillUnitIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.mixerWagon#fillUnitIndex"), 1)
            
    -- remove grass_windrow from fillTypes, we do not want to support grass in mixer wagons becasue this would make hay "useless"
    --Could i just remove this section entirely? Rather than altering the function?
    local fillUnit = self:getFillUnitByIndex(spec.fillUnitIndex)
    if fillUnit ~= nil then
        fillUnit.needsSaving = false
        if fillUnit.supportedFillTypes[FillType.GRASS_WINDROW] then
            fillUnit.needsSaving = true -- or maybe this bit? no 'if' arguement?
        end
    end
            
    -- disable sync of fillLevels for fillUnit - will be handled by mixerWagon
    fillUnit.synchronizeFillLevel = false

    spec.mixerWagonFillTypes = {}
    spec.fillTypeToMixerWagonFillType = {}
    local sumRatio = 0
    local i=0
    while true do
        local baseName = string.format("vehicle.mixerWagon.mixerWagonFillTypes.mixerWagonFillType(%d)", i)
        if not hasXMLProperty(self.xmlFile, baseName) then
            break
        end
        local entry = {}
        entry.fillTypes = {}

        local j=0
        while true do
            local fillTypeKey = baseName..string.format(".fillType(%d)", j)
            if not hasXMLProperty(self.xmlFile, fillTypeKey) then
                break
            end
            local fillTypeStr = getXMLString(self.xmlFile, fillTypeKey.."#fillType")
            if fillTypeStr ~= nil then
                local fillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeStr)
                -- we do not want to support grass in mixer wagons becasue this would make hay "useless"
                -- while i wont change this in game i do need it to support grass for my mod! (this is correctly done)
                if fillTypeIndex ~= nil then
                    if spec.fillTypeToMixerWagonFillType[fillTypeIndex] == nil then
                        entry.fillTypes[fillTypeIndex] = true
                        spec.fillTypeToMixerWagonFillType[fillTypeIndex] = entry
                    else
                        g_logManager:xmlWarning(self.configFileName, "MixerWagonFillType '%s' in '%s' already used! Ignoring it!", fillTypeKey, fillTypeStr)
                    end;
                else
                    g_logManager:xmlWarning(self.configFileName, "FillType '%s' not defined for mixerWagonFillType '%s'!", fillTypeStr, fillTypeKey)
                end
            end

            j = j+1
        end
                
        entry.name = Utils.getNoNil(getXMLString(self.xmlFile, baseName.."#name"), "unknown")
        entry.minPercentage = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#minPercentage"), 0)*0.01
        entry.maxPercentage = Utils.getNoNil(getXMLFloat(self.xmlFile, baseName.."#maxPercentage"), 100)*0.01
        entry.ratio = entry.maxPercentage  - entry.minPercentage
        entry.fillLevel = 0

        if next(entry.fillTypes) ~= nil then
            sumRatio = sumRatio + entry.ratio
            table.insert(spec.mixerWagonFillTypes, entry)
        end
        i = i+1
    end
            
    for i, entry in ipairs(spec.mixerWagonFillTypes) do
        entry.ratio = entry.ratio / sumRatio
    end
            
    spec.dirtyFlag = self:getNextDirtyFlag()
            
    if savegame ~= nil then
        for i, entry in ipairs(spec.mixerWagonFillTypes) do
            local fillTypeKey = savegame.key..string.format(".mixerWagon.fillType(%d)#fillLevel", i-1)
            local fillLevel = Utils.getNoNil(getXMLFloat(savegame.xmlFile, fillTypeKey), 0)
                    
            if fillLevel > 0 then
                self:addFillUnitFillLevel(self:getOwnerFarmId(), spec.fillUnitIndex, fillLevel, next(entry.fillTypes), ToolType.UNDEFINED, nil)
            end
        end
    end
end